use clap::{arg, command, value_parser, ArgAction};
use std::path::PathBuf;

// https://docs.rs/clap/latest/clap/_tutorial/chapter_0/index.html

use sudoku::ksudoku::{load_ksudoku, save_ksudoku};
use sudoku::sudoku::*;

fn main() {
    let mut s = Sudoku::new();

    let args = command!()
        .arg(
            arg!(-f --file <FILE> "Filename to load")
                .required(false)
                .value_parser(value_parser!(PathBuf)),
        )
        .arg(
            arg!(
                -m --make ... "Make a new sudoku puzzle"
            )
            .action(ArgAction::SetTrue),
        )
        .arg(arg!(-b --brute ... "Brute Force solver").action(ArgAction::SetTrue))
        .arg(arg!(-d --debug ... "Debug output").action(ArgAction::SetTrue))
        .get_matches();

    let mut debug: bool = false;
    if args.get_flag("debug") {
        debug = true;
    }

    if args.get_flag("make") {
        s.make();
        let mut p = s.clone();

        // Ok, get working on the puzzle.
        for _x in 0..81 {
            while p.remove() {
                print!("-");
            }
        }
        println!("");

        p.display();
        println!("Solution:");
        s.display();
        if let Some(filename) = args.get_one::<PathBuf>("file") {
            let puz = p.save_to_tld('b', '_');
            let sol = s.save_to_tld('b', '_');
            let r = save_ksudoku(filename.to_path_buf(), &puz, &sol);
            if r.is_ok() {
                println!("Saved!");
            }
        }
        return;
    }

    if let Some(filename) = args.get_one::<PathBuf>("file") {
        let puzzle = load_ksudoku(filename.to_path_buf()).unwrap();
        // Ksudoku is stored TLD.
        s.load_from_tld('b', '_', puzzle.as_str());
        s.display();

        if args.get_flag("brute") {
            println!("Solutions: {}", s.bruteforce_solver());
        } else {
            if debug {
                s.display_possible();
            }

            while s.solve(debug) {
                println!("Try it again...");
                if debug {
                    s.display();
                    s.display_possible();
                }
            }

            if !s.puzzle_complete() {
                println!("Failed to solve by logic alone.  Trying bruteforce.");
                println!("Bruteforce solutions: {}", s.bruteforce_solver());
            }

            if !debug {
                s.display();
            }
        }
    }
}
